<?php

/*******************************************************************************
  The MIT License

  Copyright (c) 2008-2009 Shark++ Software.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

 ==============================================================================
  参考として The MIT License の日本語訳を下記に併記しますが、頒布条件としては、
  上記原文に従ってください。
 ==============================================================================

  The MIT License

  Copyright (c) 2008-2009 Shark++ Software.

  以下に定める条件に従い、本ソフトウェアおよび関連文書のファイル
  （以下「ソフトウェア」）の複製を取得するすべての人に対し、ソフトウェアを
  無制限に扱うことを無償で許可します。これには、ソフトウェアの複製を使用、
  複写、変更、結合、掲載、頒布、サブライセンス、および/または販売する権利、
  およびソフトウェアを提供する相手に同じことを許可する権利も無制限に含まれ
  ます。

  上記の著作権表示および本許諾表示を、ソフトウェアのすべての複製または重要
  な部分に記載するものとします。

  ソフトウェアは「現状のまま」で、明示であるか暗黙であるかを問わず、何らの
  保証もなく提供されます。ここでいう保証とは、商品性、特定の目的への適合性、
  および権利非侵害についての保証も含みますが、それに限定されるものではあり
  ません。作者または著作権者は、契約行為、不法行為、またはそれ以外であろう
  と、ソフトウェアに起因または関連し、あるいはソフトウェアの使用またはその
  他の扱いによって生じる一切の請求、損害、その他の義務について何らの責任も
  負わないものとします。 
 *******************************************************************************/

class TrackbackController extends PluginController
{
	// コンストラクタ
	function __construct()
	{
		AuthUser::load();
		if( !AuthUser::isLoggedIn() ) {
			redirect(get_url('login'));
		}

		$this->setLayout('backend');

		if( version_compare(FROG_VERSION, '0.9.4', '<=') )
		{
			$this->assignToLayout('sidebar', new View('../../../plugins/trackback/views/sidebar'));
		}
		else
		{
			$this->assignToLayout('sidebar', new View('../../plugins/trackback/views/sidebar'));
		}
	}

	// インデックスページ(trackbackへリダイレクト)
	function index()
	{
		redirect(get_url('plugin/trackback/trackback'));
	}

	// トラックバックページ
	//  $page : ページ番号(1～)
	function trackback($page = '1')
	{
		$page = intval($page) - 1;

		$trackback_count = Trackback::count('is_approved=?', array(1));
		$rows_per_page   = PluginTrackback::getSetting('rows_per_page');
		$page_num        = intval(($trackback_count + $rows_per_page - 1) / $rows_per_page);

		if( $page < 0 ) {
			$page = 0;
		}
		if( $page_num <= $page ) {
			$page = $page_num - 1;
		}

		$this->display('trackback/views/index'
			, array(
					'page'          => $page,
					'page_num'      => $page_num,
					'rows_per_page' => PluginTrackback::getSetting('rows_per_page'),
					'trackback'     =>
						Trackback::findAll(
							'is_approved=? ORDER BY created_on DESC '.
								'LIMIT '.($page * $rows_per_page).', '.$rows_per_page.'',
							array(1)
						),
				)
			);
	}

	// モデレーション(認証待ち)ページ
	//  $page : ページ番号(1～)
	function moderation($page = '1')
	{
		$page = intval($page) - 1;

		$trackback_count = Trackback::count('is_approved=?', array(0));
		$rows_per_page   = PluginTrackback::getSetting('rows_per_page');
		$page_num        = intval(($trackback_count + $rows_per_page - 1) / $rows_per_page);

		if( $page < 0 ) {
			$page = 0;
		}
		if( $page_num <= $page ) {
			$page = $page_num - 1;
		}

		$this->display('trackback/views/moderation'
			, array(
					'page'          => $page,
					'page_num'      => $page_num,
					'rows_per_page' => PluginTrackback::getSetting('rows_per_page'),
					'trackback'     =>
						Trackback::findAll(
							'is_approved=? ORDER BY created_on DESC '.
								'LIMIT '.($page * $rows_per_page).', '.$rows_per_page.'',
							array(1)
						),
				)
			);
	}

	// 設定ページ
	function settings()
	{
		if( isset($_POST['commit']) )
		{
			$check_list =
				array(
					'auto_approve'        => array('validator' => create_function('$v', 'return false !== array_search($v, array("0", "1"));'),
					                               'error'     => __('Invalid setting value.')),
					'ping_title'          => array('validator' => create_function('$v', 'return true;'),
					                               'error'     => __('Invalid setting value.')),
					'rows_per_page'       => array('validator' => create_function('$v', 'return is_numeric($v) && 0 < intval($v);'),
					                               'error'     => __('Please specify one or more for a number of track backs of each pages.')),
					'default_status_id'   => array('validator' => create_function('$v', 'return is_numeric($v) && 0 <= intval($v) && intval($v) <= 2;'),
					                               'error'     => __('Invalid setting value.')),
					'spam_block_keyword'  => array('validator' => create_function('$v', 'return true;'),
					                               'error'     => __('Invalid setting value.')),
					'display_count_in_tab'=> array('validator' => create_function('$v', 'return false !== array_search($v, array("0", "1"));'),
					                               'error'     => __('Invalid setting value.')),
				);
			$check_result = true;
			foreach($check_list as $check_key => $validate)
			{
				if( !array_key_exists($check_key, $_POST) ||
					!$validate['validator']($_POST[$check_key]) )
				{
					Flash::set('error', $validate['error']/* , array(':field' => $check_key) */);
					$check_result = false;
					break;
				}
			}

			if( $check_result )
			{
				PluginTrackback::setAllSettings(
					array(
						'auto_approve'         => $_POST['auto_approve'],
						'ping_title'           => $_POST['ping_title'],
						'rows_per_page'        => $_POST['rows_per_page'],
						'default_status_id'    => $_POST['default_status_id'],
						'spam_block_keyword'   => $_POST['spam_block_keyword'],
						'display_count_in_tab' => $_POST['display_count_in_tab'],
					));
				Flash::set('success', __('Settings has been saved!'));
			}

			redirect(get_url('plugin/trackback/settings'));
		}

		$this->display(
				'trackback/views/settings',
				array(
					'auto_approve'         => PluginTrackback::getSetting('auto_approve'),
					'ping_title'           => PluginTrackback::getSetting('ping_title'),
					'rows_per_page'        => PluginTrackback::getSetting('rows_per_page'),
					'default_status_id'    => PluginTrackback::getSetting('default_status_id'),
					'spam_block_keyword'   => PluginTrackback::getSetting('spam_block_keyword'),
					'display_count_in_tab' => PluginTrackback::getSetting('display_count_in_tab'),
				)
			);
	}

	// ドキュメントページ
	function documentation()
	{
		$this->display('trackback/views/documentation');
	}

	function approve($id)
	{
		if( isset($id) ) {
			Record::update('Trackback', array('is_approved' => '1'), 'id=?', array($id));
		}

		Flash::set('success', __('Trackback has been approved!'));

		redirect(get_url('plugin/trackback/moderation'));
	}

	function reject($id)
	{
		if( isset($id) ) {
			Record::update('Trackback', array('is_approved' => '0'), 'id=?', array($id));
		}

		Flash::set('success', __('Trackback has been unapproved!'));

		redirect(get_url('plugin/trackback/trackback'));
	}

	function edit($id)
	{
	}

	function delete($id)
	{
		if( isset($id) ) {
			$trackback = Trackback::findById($id);
			if( isset($id) ) {
				$trackback->delete();
			}
		}

		Flash::set('success', __('Trackback has been deleted!'));

		redirect(get_url('plugin/trackback/trackback'));
	}
}

